@@ -4,6 +4,7 @@
 
 
 setClass("DelayedArray",
+    contains="Array",
     representation(
         seed="ANY",              # An array-like object expected to satisfy
                                  # the "seed contract" i.e. to support dim(),
@@ -60,38 +61,34 @@ setMethod("matrixClass", "DelayedArray", function(x) "DelayedMatrix")
 ### Validity
 ###
 
-.wmsg2 <- function(...)
-    paste0("\n    ",
-           paste0(strwrap(paste0(c(...), collapse="")), collapse="\n    "))
-
 .validate_DelayedArray <- function(x)
 {
     x_dim <- dim(x@seed)
     x_ndim <- length(x_dim)
     ## 'seed' slot.
     if (x_ndim == 0L)
-        return(.wmsg2("'x@seed' must have dimensions"))
+        return(wmsg2("'x@seed' must have dimensions"))
     ## 'index' slot.
     if (length(x@index) != x_ndim)
-        return(.wmsg2("'x@index' must have one list element per dimension ",
-                      "in 'x@seed'"))
+        return(wmsg2("'x@index' must have one list element per dimension ",
+                     "in 'x@seed'"))
     if (!all(S4Vectors:::sapply_isNULL(x@index) |
              vapply(x@index, is.integer, logical(1), USE.NAMES=FALSE)))
-        return(.wmsg2("every list element in 'x@index' must be either NULL ",
-                      "or an integer vector"))
+        return(wmsg2("every list element in 'x@index' must be either NULL ",
+                     "or an integer vector"))
     ## 'metaindex' slot.
     if (length(x@metaindex) == 0L)
-        return(.wmsg2("'x@metaindex' cannot be empty"))
+        return(wmsg2("'x@metaindex' cannot be empty"))
     if (S4Vectors:::anyMissingOrOutside(x@metaindex, 1L, x_ndim))
-        return(.wmsg2("all values in 'x@metaindex' must be >= 1 ",
-                      "and <= 'length(x@index)'"))
+        return(wmsg2("all values in 'x@metaindex' must be >= 1 ",
+                     "and <= 'length(x@index)'"))
     if (!isStrictlySorted(x@metaindex))
-        return(.wmsg2("'x@metaindex' must be strictly sorted"))
+        return(wmsg2("'x@metaindex' must be strictly sorted"))
     if (!all(get_Nindex_lengths(x@index, x_dim)[-x@metaindex] == 1L))
-        return(.wmsg2("all the dropped dimensions in 'x' must be equal to 1"))
+        return(wmsg2("all the dropped dimensions in 'x' must be equal to 1"))
     ## 'is_transposed' slot.
     if (!isTRUEorFALSE(x@is_transposed))
-        return(.wmsg2("'x@is_transposed' must be TRUE or FALSE"))
+        return(wmsg2("'x@is_transposed' must be TRUE or FALSE"))
     TRUE
 }
 
@@ -102,7 +99,7 @@ setValidity2("DelayedArray", .validate_DelayedArray)
 .validate_DelayedMatrix <- function(x)
 {
     if (length(dim(x)) != 2L)
-        return(wmsg("'x' must have exactly 2 dimensions"))
+        return(wmsg2("'x' must have exactly 2 dimensions"))
     TRUE
 }
 
@@ -201,11 +198,6 @@ downgrade_to_DelayedArray_or_DelayedMatrix <- function(x)
 
 setMethod("dim", "DelayedArray", .get_DelayedArray_dim)
 
-### Even though prod() always returns a double, it seems that the length()
-### primitive function automatically turns this double into an integer if
-### it's <= .Machine$integer.max
-setMethod("length", "DelayedArray", function(x) prod(dim(x)))
-
 setMethod("isEmpty", "DelayedArray", function(x) any(dim(x) == 0L))
 
 ### dim() setter.
@@ -402,8 +394,8 @@ setMethod("names", "DelayedArray", .get_DelayedArray_names)
 {
     if (length(dim(x)) != 1L) {
         if (!is.null(value))
-            stop("setting the names of a ", class(x), " object with more ",
-                 "than 1 dimension is not supported")
+            stop(wmsg("setting the names of a ", class(x), " object ",
+                      "with more than 1 dimension is not supported"))
         return(x)
     }
     dimnames(x)[[1L]] <- value
@@ -414,258 +406,18 @@ setReplaceMethod("names", "DelayedArray", .set_DelayedArray_names)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### [
+### Transpose
 ###
 
-### Return an object of the same class as 'x' (endomorphism).
-### 'user_Nindex' must be a "multidimensional subsetting Nindex" i.e. a
-### list with one subscript per dimension in 'x'. Missing subscripts are
-### represented by NULLs.
-.subset_DelayedArray_by_Nindex <- function(x, user_Nindex)
-{
-    stopifnot(is.list(user_Nindex))
-    x_index <- x@index
-    x_ndim <- length(x@metaindex)
-    x_seed_dim <- dim(seed(x))
-    x_seed_dimnames <- dimnames(seed(x))
-    x_delayed_ops <- x@delayed_ops
-    for (n in seq_along(user_Nindex)) {
-        subscript <- user_Nindex[[n]]
-        if (is.null(subscript))
-            next
-        n0 <- if (x@is_transposed) x_ndim - n + 1L else n
-        N <- x@metaindex[[n0]]
-        i <- x_index[[N]]
-        if (is.null(i)) {
-            i <- seq_len(x_seed_dim[[N]])  # expand 'i'
-            names(i) <- get_Nindex_names_along(x_index, x_seed_dimnames, N)
-        }
-        subscript <- normalizeSingleBracketSubscript(subscript, i,
-                                                     as.NSBS=TRUE)
-        x_index[[N]] <- extractROWS(i, subscript)
-        if (n0 == 1L)
-            x_delayed_ops <- .subset_delayed_ops_args(x_delayed_ops, subscript,
-                                                      FALSE)
-        if (n0 == x_ndim)
-            x_delayed_ops <- .subset_delayed_ops_args(x_delayed_ops, subscript,
-                                                      TRUE)
-    }
-    if (!identical(x@index, x_index)) {
+### The actual transposition of the data is delayed i.e. it will be realized
+### on the fly only when as.array() (or as.vector() or as.matrix()) is called
+### on 'x'.
+setMethod("t", "DelayedArray",
+    function(x)
+    {
         x <- downgrade_to_DelayedArray_or_DelayedMatrix(x)
-        x@index <- x_index
-        if (!identical(x@delayed_ops, x_delayed_ops))
-            x@delayed_ops <- x_delayed_ops
-    }
-    x
-}
-
-### Return an atomic vector.
-.subset_DelayedArray_by_1Dindex <- function(x, i)
-{
-    if (!is.numeric(i))
-        stop(wmsg("1D-style subsetting of a DelayedArray object only ",
-                  "accepts a numeric subscript at the moment"))
-    if (length(i) == 0L) {
-        user_Nindex <- as.list(integer(length(dim(x))))
-        ## x0 <- x[0, ..., 0]
-        x0 <- .subset_DelayedArray_by_Nindex(x, user_Nindex)
-        return(as.vector(x0))
-    }
-    if (anyNA(i))
-        stop(wmsg("1D-style subsetting of a DelayedArray object does ",
-                  "not support NA indices yet"))
-    if (min(i) < 1L)
-        stop(wmsg("1D-style subsetting of a DelayedArray object only ",
-                  "supports positive indices at the moment"))
-    if (max(i) > length(x))
-        stop(wmsg("subscript contains out-of-bounds indices"))
-    if (length(i) == 1L)
-        return(.get_DelayedArray_element(x, i))
-
-    ## We want to walk only on the blocks that we actually need to visit so we
-    ## don't use block_APPLY() or family because they walk on all the blocks.
-
-    block_len <- get_block_length(type(x))
-    blocks <- ArrayBlocks(dim(x), block_len)
-    nblock <- length(blocks)
-
-    breakpoints <- cumsum(get_block_lengths(blocks))
-    part_idx <- get_part_index(i, breakpoints)
-    split_part_idx <- split_part_index(part_idx, length(breakpoints))
-    block_idx <- which(lengths(split_part_idx) != 0L)  # blocks to visit
-    res <- lapply(block_idx, function(b) {
-            if (get_verbose_block_processing())
-                message("Visiting block ", b, "/", nblock, " ... ",
-                        appendLF=FALSE)
-            Nindex <- get_array_block_Nindex(blocks, b)
-            subarray <- subset_by_Nindex(x, Nindex)
-            if (!is.array(subarray))
-                subarray <- as.array(subarray)
-            block_ans <- subarray[split_part_idx[[b]]]
-            if (get_verbose_block_processing())
-                message("OK")
-            block_ans
-    })
-    unlist(res, use.names=FALSE)[get_rev_index(part_idx)]
-}
-
-.subset_DelayedArray <- function(x, i, j, ..., drop=TRUE)
-{
-    if (missing(x))
-        stop("'x' is missing")
-
-    ## Check the dimensionality of the user call i.e whether the function was
-    ## called with 1D-style, or 2D-style, or 3D-style etc... subsetting.
-    ndim <- nargs() - 1L
-    x_dim <- dim(x)
-    x_ndim <- length(x_dim)
-    if (!missing(drop))
-        ndim <- ndim - 1L
-    if (ndim == 1L && missing(i))
-        ndim <- 0L
-    if (ndim != 0L && ndim != x_ndim) {
-        if (ndim != 1L)
-            stop("incorrect number of dimensions")
-        return(.subset_DelayedArray_by_1Dindex(x, i))
-    }
-
-    ## Prepare the "multidimensional subsetting index".
-    user_Nindex <- vector(mode="list", length=x_ndim)
-    if (!missing(i))
-        user_Nindex[[1L]] <- if (is.null(i)) integer(0) else i
-    if (!missing(j))
-        user_Nindex[[2L]] <- if (is.null(j)) integer(0) else j
-    dots <- match.call(expand.dots=FALSE)$...  # list of non-evaluated args
-    eframe <- parent.frame()
-    for (n2 in seq_along(dots)) {
-        k <- dots[[n2]]
-        if (!missing(k)) {
-            k <- eval(k, envir=eframe, enclos=eframe)
-            user_Nindex[[2L + n2]] <- if (is.null(k)) integer(0) else k
-        }
-    }
-
-    ## Perform the subsetting.
-    .subset_DelayedArray_by_Nindex(x, user_Nindex)
-}
-
-setMethod("[", "DelayedArray", .subset_DelayedArray)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### subset_seed_as_array()
-###
-
-### Return the slice as a list.
-.extract_data_frame_slice <- function(x, index)
-{
-    slice <- subset_by_Nindex(x, index)
-    ## Turn into a list and replace factors with character vectors.
-    lapply(slice, as.vector)
-}
-.extract_DataFrame_slice <- function(x, index)
-{
-    slice <- subset_by_Nindex(x, index)
-    slice <- as.data.frame(slice)
-    ## Turn into a list and replace factors with character vectors.
-    lapply(slice, as.vector)
-}
-
-### Return a list with one list element per column in data frame 'x'.
-### All the list elements have length 0.
-.extract_data_frame_slice0 <- function(x)
-{
-    slice0 <- x[0L, , drop=FALSE]
-    ## Turn into a list and replace factors with character vectors.
-    lapply(slice0, as.vector)
-}
-.extract_DataFrame_slice0 <- function(x)
-{
-    slice0 <- x[0L, , drop=FALSE]
-    slice0 <- as.data.frame(slice0)
-    if (ncol(slice0) != ncol(x))
-        stop(wmsg("DataFrame object 'x' can be used as the seed of ",
-                  "a DelayedArray object only if as.data.frame(x) ",
-                  "preserves the number of columns"))
-    ## Turn into a list and replace factors with character vectors.
-    lapply(slice0, as.vector)
-}
-
-### Equivalent to 'typeof(as.matrix(x))' but with an almost-zero
-### memory footprint (it avoids the cost of turning 'x' into a matrix).
-.get_data_frame_type <- function(x)
-{
-    if (ncol(x) == 0L)
-        return("logical")
-    slice0 <- .extract_data_frame_slice0(x)
-    typeof(unlist(slice0, use.names=FALSE))
-}
-
-### Equivalent to 'typeof(as.matrix(as.data.frame(x)))' but with an
-### almost-zero memory footprint (it avoids the cost of turning 'x' first
-### into a data frame then into a matrix).
-.get_DataFrame_type <- function(x)
-{
-    if (ncol(x) == 0L)
-        return("logical")
-    slice0 <- .extract_DataFrame_slice0(x)
-    typeof(unlist(slice0, use.names=FALSE))
-}
-
-### 'index' is expected to be an unnamed list of subscripts as positive integer
-### vectors, one vector per seed dimension. *Missing* list elements are allowed
-### and represented by NULLs.
-### The "subset_seed_as_array" methods don't need to support anything else.
-### They must return an ordinary array. No need to propagate the dimnames.
-setGeneric("subset_seed_as_array", signature="seed",
-    function(seed, index) standardGeneric("subset_seed_as_array")
-)
-
-setMethod("subset_seed_as_array", "ANY",
-    function(seed, index)
-    {
-        slice <- subset_by_Nindex(seed, index)
-        as.array(slice)
-    }
-)
-
-setMethod("subset_seed_as_array", "array",
-    function(seed, index)
-        subset_by_Nindex(seed, index)
-)
-
-### Equivalent to
-###
-###     subset_by_Nindex(as.matrix(x), index)
-###
-### but avoids the cost of turning the full data frame 'x' into a matrix so
-### memory footprint stays small when 'index' is small.
-setMethod("subset_seed_as_array", "data.frame",
-    function(seed, index)
-    {
-        #ans_type <- .get_data_frame_type(seed)
-        slice0 <- .extract_data_frame_slice0(seed)
-        slice <- .extract_data_frame_slice(seed, index)
-        data <- unlist(c(slice0, slice), use.names=FALSE)
-        array(data, dim=get_Nindex_lengths(index, dim(seed)))
-    }
-)
-
-### Equivalent to
-###
-###     subset_by_Nindex(as.matrix(as.data.frame(x)), index)
-###
-### but avoids the cost of turning the full DataFrame 'x' first into a data
-### frame then into a matrix so memory footprint stays small when 'index' is
-### small.
-setMethod("subset_seed_as_array", "DataFrame",
-    function(seed, index)
-    {
-        #ans_type <- .get_DataFrame_type(seed)
-        slice0 <- .extract_DataFrame_slice0(seed)
-        slice <- .extract_DataFrame_slice(seed, index)
-        data <- unlist(c(slice0, slice), use.names=FALSE)
-        array(data, dim=get_Nindex_lengths(index, dim(seed)))
+        x@is_transposed <- !x@is_transposed
+        x
     }
 )
 
@@ -800,27 +552,225 @@ register_delayed_op <- function(x, FUN, Largs=list(), Rargs=list(),
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Transpose
+### Multi-dimensional single bracket subsetting
+###
+###     x[i_1, i_2, ..., i_n]
+###
+### Return an object of the same class as 'x' (endomorphism).
+### 'user_Nindex' must be a "multidimensional subsetting Nindex" i.e. a
+### list with one subscript per dimension in 'x'. Missing subscripts must
+### be represented by NULLs.
 ###
 
-### The actual transposition of the data is delayed i.e. it will be realized
-### on the fly only when as.array() (or as.vector() or as.matrix()) is called
-### on 'x'.
-setMethod("t", "DelayedArray",
+.subset_DelayedArray_by_Nindex <- function(x, user_Nindex)
+{
+    stopifnot(is.list(user_Nindex))
+    x_index <- x@index
+    x_ndim <- length(x@metaindex)
+    x_seed_dim <- dim(seed(x))
+    x_seed_dimnames <- dimnames(seed(x))
+    x_delayed_ops <- x@delayed_ops
+    for (n in seq_along(user_Nindex)) {
+        subscript <- user_Nindex[[n]]
+        if (is.null(subscript))
+            next
+        n0 <- if (x@is_transposed) x_ndim - n + 1L else n
+        N <- x@metaindex[[n0]]
+        i <- x_index[[N]]
+        if (is.null(i)) {
+            i <- seq_len(x_seed_dim[[N]])  # expand 'i'
+            names(i) <- get_Nindex_names_along(x_index, x_seed_dimnames, N)
+        }
+        subscript <- normalizeSingleBracketSubscript(subscript, i,
+                                                     as.NSBS=TRUE)
+        x_index[[N]] <- extractROWS(i, subscript)
+        if (n0 == 1L)
+            x_delayed_ops <- .subset_delayed_ops_args(x_delayed_ops, subscript,
+                                                      FALSE)
+        if (n0 == x_ndim)
+            x_delayed_ops <- .subset_delayed_ops_args(x_delayed_ops, subscript,
+                                                      TRUE)
+    }
+    if (!identical(x@index, x_index)) {
+        x <- downgrade_to_DelayedArray_or_DelayedMatrix(x)
+        x@index <- x_index
+        if (!identical(x@delayed_ops, x_delayed_ops))
+            x@delayed_ops <- x_delayed_ops
+    }
+    x
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### type()
+###
+### For internal use only.
+###
+
+setGeneric("type", function(x) standardGeneric("type"))
+
+setMethod("type", "array", function(x) typeof(x))
+
+### If 'x' is a DelayedArray object, 'type(x)' must always return the same
+### as 'typeof(as.array(x))'.
+setMethod("type", "DelayedArray",
     function(x)
     {
-        x <- downgrade_to_DelayedArray_or_DelayedMatrix(x)
-        x@is_transposed <- !x@is_transposed
-        x
+        user_Nindex <- as.list(integer(length(dim(x))))
+        ## x0 <- x[0, ..., 0]
+        x0 <- .subset_DelayedArray_by_Nindex(x, user_Nindex)
+        typeof(as.array(x0, drop=TRUE))
     }
 )
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### as.array()
+### Linear single bracket subsetting
+###
+###     x[i]
+###
+### Return an atomic vector.
 ###
 
-### TODO: Not sure we need this. Using drop() should do it.
+.subset_DelayedArray_by_1Dindex <- function(x, i)
+{
+    if (!is.numeric(i))
+        stop(wmsg("1D-style subsetting of a DelayedArray object only ",
+                  "accepts a numeric subscript at the moment"))
+    if (length(i) == 0L) {
+        user_Nindex <- as.list(integer(length(dim(x))))
+        ## x0 <- x[0, ..., 0]
+        x0 <- .subset_DelayedArray_by_Nindex(x, user_Nindex)
+        return(as.vector(x0))
+    }
+    if (anyNA(i))
+        stop(wmsg("1D-style subsetting of a DelayedArray object does ",
+                  "not support NA indices yet"))
+    if (min(i) < 1L)
+        stop(wmsg("1D-style subsetting of a DelayedArray object only ",
+                  "supports positive indices at the moment"))
+    if (max(i) > length(x))
+        stop(wmsg("subscript contains out-of-bounds indices"))
+    if (length(i) == 1L)
+        return(.get_DelayedArray_element(x, i))
+
+    ## We want to walk only on the blocks that we actually need to visit so we
+    ## don't use block_APPLY() or family because they walk on all the blocks.
+
+    max_block_len <- get_max_block_length(type(x))
+    spacings <- get_max_spacings_for_linear_blocks(dim(x), max_block_len)
+    grid <- ArrayRegularGrid(dim(x), spacings)
+    nblock <- length(grid)
+
+    breakpoints <- cumsum(lengths(grid))
+    part_idx <- get_part_index(i, breakpoints)
+    split_part_idx <- split_part_index(part_idx, length(breakpoints))
+    block_idx <- which(lengths(split_part_idx) != 0L)  # blocks to visit
+    res <- lapply(block_idx, function(b) {
+            if (get_verbose_block_processing())
+                message("Visiting block ", b, "/", nblock, " ... ",
+                        appendLF=FALSE)
+            block <- extract_block(x, grid[[b]])
+            if (!is.array(block))
+                block <- as.array(block)
+            block_ans <- block[split_part_idx[[b]]]
+            if (get_verbose_block_processing())
+                message("OK")
+            block_ans
+    })
+    unlist(res, use.names=FALSE)[get_rev_index(part_idx)]
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### [
+###
+
+.subset_DelayedArray <- function(x, i, j, ..., drop=TRUE)
+{
+    if (missing(x))
+        stop("'x' is missing")
+    user_Nindex <- extract_Nindex_from_syscall(sys.call(), parent.frame())
+    nsubscript <- length(user_Nindex)
+    if (nsubscript != 0L && nsubscript != length(dim(x))) {
+        if (nsubscript != 1L)
+            stop("incorrect number of subscripts")
+        return(.subset_DelayedArray_by_1Dindex(x, user_Nindex[[1L]]))
+    }
+    .subset_DelayedArray_by_Nindex(x, user_Nindex)
+}
+setMethod("[", "DelayedArray", .subset_DelayedArray)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### [<-
+###
+
+.filling_error_msg <- c(
+    "filling a DelayedArray object 'x' with a value (i.e. 'x[] <- value') ",
+    "is supported only when 'value' is an atomic vector and 'length(value)' ",
+    "is a divisor of 'nrow(x)'"
+)
+
+.subassign_error_msg <- c(
+    "subassignment to a DelayedArray object 'x' (i.e. 'x[i] <- value') is ",
+    "supported only when the subscript 'i' is a logical DelayedArray object ",
+    "with the same dimensions as 'x' and when 'value' is a scalar (i.e. an ",
+    "atomic vector of length 1)"
+)
+
+.fill_DelayedArray_with_value <- function(x, value)
+{
+    if (!(is.vector(value) && is.atomic(value)))
+        stop(wmsg(.filling_error_msg))
+    value_len <- length(value)
+    if (value_len == 1L)
+        return(register_delayed_op(x, `[<-`, Rargs=list(value=value)))
+    x_len <- length(x)
+    if (value_len > x_len)
+        stop(wmsg("'value' is longer than 'x'"))
+    x_nrow <- nrow(x)
+    if (x_nrow != 0L) {
+        if (value_len == 0L || x_nrow %% value_len != 0L)
+            stop(wmsg(.filling_error_msg))
+        value <- rep(value, length.out=x_nrow)
+    }
+    register_delayed_op(x, `[<-`, Rargs=list(value=value),
+                                  recycle_along_last_dim=x@is_transposed)
+}
+
+.subassign_DelayedArray <- function(x, i, j, ..., value)
+{
+    if (missing(x))
+        stop("'x' is missing")
+    user_Nindex <- extract_Nindex_from_syscall(sys.call(), parent.frame())
+    nsubscript <- length(user_Nindex)
+    if (nsubscript == 0L)
+        return(.fill_DelayedArray_with_value(x, value))
+    if (nsubscript != 1L)
+        stop(wmsg(.subassign_error_msg))
+    i <- user_Nindex[[1L]]
+    if (!(is(i, "DelayedArray") &&
+          identical(dim(x), dim(i)) &&
+          type(i) == "logical"))
+        stop(wmsg(.subassign_error_msg))
+    if (!(is.vector(value) && is.atomic(value) && length(value) == 1L))
+        stop(wmsg(.subassign_error_msg))
+    DelayedArray(new_ConformableSeedCombiner(x, i,
+                                             COMBINING_OP=`[<-`,
+                                             Rargs=list(value=value)))
+}
+
+setReplaceMethod("[", "DelayedArray", .subassign_DelayedArray)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Realization in memory
+###
+###     as.array(x)
+###
+
+### TODO: Do we actually need this? Using drop() should do it.
 .reduce_array_dimensions <- function(x)
 {
     x_dim <- dim(x)
@@ -956,29 +906,6 @@ setAs("DelayedMatrix", "dgCMatrix", .from_DelayedMatrix_to_dgCMatrix)
 setAs("DelayedMatrix", "sparseMatrix", .from_DelayedMatrix_to_dgCMatrix)
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### type()
-###
-### For internal use only.
-###
-
-setGeneric("type", function(x) standardGeneric("type"))
-
-setMethod("type", "array", function(x) typeof(x))
-
-### If 'x' is a DelayedArray object, 'type(x)' must always return the same
-### as 'typeof(as.array(x))'.
-setMethod("type", "DelayedArray",
-    function(x)
-    {
-        user_Nindex <- as.list(integer(length(dim(x))))
-        ## x0 <- x[0, ..., 0]
-        x0 <- .subset_DelayedArray_by_Nindex(x, user_Nindex)
-        typeof(as.array(x0, drop=TRUE))
-    }
-)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### [[
 ###
@@ -1016,30 +943,14 @@ setMethod("show", "DelayedArray", show_compact_array)
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Combining and splitting
 ###
-### Combining arrays with c() is NOT an endomorphism!
-###
-
-### 'objects' must be a list of array-like objects that support as.vector().
-combine_array_objects <- function(objects)
-{
-    if (!is.list(objects))
-        stop("'objects' must be a list")
-
-    NULL_idx <- which(S4Vectors:::sapply_isNULL(objects))
-    if (length(NULL_idx) != 0L)
-        objects <- objects[-NULL_idx]
-    if (length(objects) == 0L)
-        return(NULL)
-
-    unlist(lapply(objects, as.vector), recursive=FALSE, use.names=FALSE)
-}
 
+### Note that combining arrays with c() is NOT an endomorphism!
 setMethod("c", "DelayedArray",
     function (x, ..., recursive=FALSE)
     {
         if (!identical(recursive, FALSE))
-            stop("\"c\" method for DelayedArray objects ",
-                 "does not support the 'recursive' argument")
+            stop(wmsg("\"c\" method for DelayedArray objects ",
+                      "does not support the 'recursive' argument"))
         if (missing(x)) {
             objects <- list(...)
         } else {
@@ -1059,3 +970,68 @@ split.DelayedArray <- function(x, f, drop=FALSE, ...)
     splitAsList(x, f, drop=drop, ...)
 setMethod("split", c("DelayedArray", "ANY"), split.DelayedArray)
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Binding
+###
+### We only support binding DelayedArray objects along the rows or the cols
+### at the moment. No binding along an arbitrary dimension yet! (i.e. no
+### "abind" method yet)
+###
+
+### arbind() and acbind()
+
+.DelayedArray_arbind <- function(...)
+{
+    objects <- unname(list(...))
+    dims <- get_dims_to_bind(objects, 1L)
+    if (is.character(dims))
+        stop(wmsg(dims))
+    DelayedArray(new_SeedBinder(objects, 1L))
+}
+
+.DelayedArray_acbind <- function(...)
+{
+    objects <- unname(list(...))
+    dims <- get_dims_to_bind(objects, 2L)
+    if (is.character(dims))
+        stop(wmsg(dims))
+    DelayedArray(new_SeedBinder(objects, 2L))
+}
+
+setMethod("arbind", "DelayedArray", .DelayedArray_arbind)
+setMethod("acbind", "DelayedArray", .DelayedArray_acbind)
+
+### rbind() and cbind()
+
+setMethod("rbind", "DelayedMatrix", .DelayedArray_arbind)
+setMethod("cbind", "DelayedMatrix", .DelayedArray_acbind)
+
+.as_DelayedMatrix_objects <- function(objects)
+{
+    lapply(objects,
+        function(object) {
+            if (length(dim(object)) != 2L)
+                stop(wmsg("cbind() and rbind() are not supported on ",
+                          "DelayedArray objects that don't have exactly ",
+                          "2 dimensions. Please use acbind() or arnind() ",
+                          "instead."))
+            as(object, "DelayedMatrix")
+        })
+}
+
+.DelayedArray_rbind <- function(...)
+{
+    objects <- .as_DelayedMatrix_objects(list(...))
+    do.call("rbind", objects)
+}
+
+.DelayedArray_cbind <- function(...)
+{
+    objects <- .as_DelayedMatrix_objects(list(...))
+    do.call("cbind", objects)
+}
+
+setMethod("rbind", "DelayedArray", .DelayedArray_rbind)
+setMethod("cbind", "DelayedArray", .DelayedArray_cbind)
+
