@@ -3,106 +3,6 @@
 ### -------------------------------------------------------------------------
 
 
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### ConformableSeedCombiner objects
-###
-### This class is for internal use only and is not exported.
-###
-
-setClass("ConformableSeedCombiner",
-    representation(
-        seeds="list",              # List of n conformable array-like objects
-                                   # to combine. Each object is expected to
-                                   # satisfy the "seed contract" i.e. to
-                                   # support dim(), dimnames(), and
-                                   # subset_seed_as_array().
-
-        COMBINING_OP="character",  # n-ary operator to combine the seeds.
-
-        Rargs="list"               # Additional arguments to the n-ary
-                                   # operator. Currently unused.
-    ),
-    prototype(
-        seeds=list(new("array")),
-        COMBINING_OP="identity"
-    )
-)
-
-.objects_are_conformable_arrays <- function(objects)
-{
-    dims <- lapply(objects, dim)
-    ndims <- lengths(dims)
-    first_ndim <- ndims[[1L]]
-    if (!all(ndims == first_ndim))
-        return(FALSE)
-    tmp <- unlist(dims, use.names=FALSE)
-    if (is.null(tmp))
-        return(FALSE)
-    dims <- matrix(tmp, nrow=first_ndim)
-    first_dim <- dims[ , 1L]
-    all(dims == first_dim)
-}
-
-.validate_ConformableSeedCombiner <- function(x)
-{
-    ## 'seeds' slot.
-    if (length(x@seeds) == 0L)
-        return(wmsg("'x@seeds' cannot be empty"))
-    if (!.objects_are_conformable_arrays(x@seeds))
-        return(wmsg("'x@seeds' must be a list of conformable ",
-                    "array-like objects"))
-    ## 'COMBINING_OP' slot.
-    if (!isSingleString(x@COMBINING_OP))
-        return(wmsg("'x@COMBINING_OP' must be a single string"))
-    OP <- try(match.fun(x@COMBINING_OP), silent=TRUE)
-    if (is(OP, "try-error"))
-        return(wmsg("the name in 'x@COMBINING_OP' (\"", x@COMBINING_OP,
-                    "\") must refer to a known n-ary operator"))
-    TRUE
-}
-
-setValidity2("ConformableSeedCombiner", .validate_ConformableSeedCombiner)
-
-.new_ConformableSeedCombiner <- function(seed=new("array"), ...,
-                                         COMBINING_OP="identity",
-                                         Rargs=list())
-{
-    seeds <- unname(list(seed, ...))
-    seeds <- lapply(seeds, remove_pristine_DelayedArray_wrapping)
-    new2("ConformableSeedCombiner", seeds=seeds,
-                                    COMBINING_OP=COMBINING_OP,
-                                    Rargs=Rargs)
-}
-
-### Implement the "seed contract" i.e. dim(), dimnames(), and
-### subset_seed_as_array().
-
-.get_ConformableSeedCombiner_dim <- function(x) dim(x@seeds[[1L]])
-
-setMethod("dim", "ConformableSeedCombiner",
-    .get_ConformableSeedCombiner_dim
-)
-
-.get_ConformableSeedCombiner_dimnames <- function(x)
-{
-    IRanges:::combine_dimnames(x@seeds)
-}
-
-setMethod("dimnames", "ConformableSeedCombiner",
-    .get_ConformableSeedCombiner_dimnames
-)
-
-.subset_ConformableSeedCombiner_as_array <- function(seed, index)
-{
-    arrays <- lapply(seed@seeds, subset_seed_as_array, index)
-    do.call(seed@COMBINING_OP, c(arrays, seed@Rargs))
-}
-
-setMethod("subset_seed_as_array", "ConformableSeedCombiner",
-    .subset_ConformableSeedCombiner_as_array
-)
-
-
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### "Ops" group generics
 ###
@@ -172,7 +72,7 @@ setMethod("subset_seed_as_array", "ConformableSeedCombiner",
 {
     if (!identical(dim(e1), dim(e2)))
         stop("non-conformable arrays")
-    DelayedArray(.new_ConformableSeedCombiner(e1, e2, COMBINING_OP=.Generic))
+    DelayedArray(new_ConformableSeedCombiner(e1, e2, COMBINING_OP=.Generic))
 }
 
 .DelayedArray_Ops <- function(.Generic, e1, e2)
@@ -263,7 +163,7 @@ setMethod("pmax2", c("ANY", "ANY"),
             names(ans) <- .combine_names(e1, e2)
         } else {
             dim(ans) <- ans_dim
-            dimnames(ans) <- IRanges:::combine_dimnames(list(e1, e2))
+            dimnames(ans) <- combine_dimnames(list(e1, e2))
         }
         ans
     }
@@ -278,7 +178,7 @@ setMethod("pmin2", c("ANY", "ANY"),
             names(ans) <- .combine_names(e1, e2)
         } else {
             dim(ans) <- ans_dim
-            dimnames(ans) <- IRanges:::combine_dimnames(list(e1, e2))
+            dimnames(ans) <- combine_dimnames(list(e1, e2))
         }
         ans
     }
@@ -400,7 +300,7 @@ setMethod("signif", "DelayedArray",
 .DelayedArray_block_anyNA <- function(x, recursive=FALSE)
 {
     APPLY <- anyNA
-    COMBINE <- function(b, subarray, init, reduced) { init || reduced }
+    COMBINE <- function(b, block, init, reduced) { init || reduced }
     init <- FALSE
     BREAKIF <- identity
 
@@ -423,13 +323,13 @@ setMethod("anyNA", "DelayedArray", .DelayedArray_block_anyNA)
     if (!isTRUEorFALSE(useNames))
         stop("'useNames' must be TRUE or FALSE")
     APPLY <- base::which
-    COMBINE <- function(b, subarray, init, reduced) {
+    COMBINE <- function(b, block, init, reduced) {
         if (length(reduced) != 0L) {
             reduced <- reduced + init[["offset"]]
             part_number <- sprintf("%010d", b)
             init[[part_number]] <- reduced
         }
-        init[["offset"]] <- init[["offset"]] + length(subarray)
+        init[["offset"]] <- init[["offset"]] + length(block)
         init
     }
     offset <- 0L
@@ -487,16 +387,16 @@ setMethod("which", "DelayedArray", .DelayedArray_block_which)
     objects <- .collect_objects(x, ...)
 
     GENERIC <- match.fun(.Generic)
-    APPLY <- function(subarray) {
-        ## We get a warning if 'subarray' is empty (which can't happen, blocks
-        ## can't be empty) or if 'na.rm' is TRUE and 'subarray' contains only
+    APPLY <- function(block) {
+        ## We get a warning if 'block' is empty (which can't happen, blocks
+        ## can't be empty) or if 'na.rm' is TRUE and 'block' contains only
         ## NA's or NaN's.
-        reduced <- tryCatch(GENERIC(subarray, na.rm=na.rm), warning=identity)
+        reduced <- tryCatch(GENERIC(block, na.rm=na.rm), warning=identity)
         if (is(reduced, "warning"))
             return(NULL)
         reduced
     }
-    COMBINE <- function(b, subarray, init, reduced) {
+    COMBINE <- function(b, block, init, reduced) {
         if (is.null(init) && is.null(reduced))
             return(NULL)
         GENERIC(init, reduced)
@@ -545,15 +445,15 @@ setMethod("Summary", "DelayedArray",
         stop("\"mean\" method for DelayedArray objects ",
              "does not support the 'trim' argument yet")
 
-    APPLY <- function(subarray) {
-        tmp <- as.vector(subarray, mode="numeric")
-        subarray_sum <- sum(tmp, na.rm=na.rm)
-        subarray_nval <- length(tmp)
+    APPLY <- function(block) {
+        tmp <- as.vector(block, mode="numeric")
+        block_sum <- sum(tmp, na.rm=na.rm)
+        block_nval <- length(tmp)
         if (na.rm)
-            subarray_nval <- subarray_nval - sum(is.na(tmp))
-        c(subarray_sum, subarray_nval)
+            block_nval <- block_nval - sum(is.na(tmp))
+        c(block_sum, block_nval)
     }
-    COMBINE <- function(b, subarray, init, reduced) { init + reduced }
+    COMBINE <- function(b, block, init, reduced) { init + reduced }
     init <- numeric(2)  # sum and nval
     BREAKIF <- function(init) is.na(init[[1L]])
 
